@@ -12,11 +12,16 @@ use Hash::FieldHash  qw( fieldhash );
 use Scalar::Util     qw( blessed reftype );
 
 BEGIN {
-	*HAS_SUB_NAME = eval { require Sub::Name } ? sub(){1} : sub(){0};
+	*HAS_SUB_UTIL = eval { require Sub::Util }
+		? sub(){1}
+		: sub(){0};
+	*HAS_SUB_NAME = !HAS_SUB_UTIL() && eval { require Sub::Name }
+		? sub(){1}
+		: sub(){0};
 };
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.007';
+our $VERSION   = '0.010';
 our @ISA       = qw/ Exporter::Tiny /;
 
 fieldhash( our %FIELDS );
@@ -32,7 +37,9 @@ sub _generate_has : method
 		$attr->install_accessors;
 	};
 	
-	$code = Sub::Name::subname("$me\::has", $code) if HAS_SUB_NAME;
+	HAS_SUB_UTIL ? ($code = Sub::Util::set_subname("$me\::has", $code)) :
+	HAS_SUB_NAME ? ($code = Sub::Name::subname("$me\::has", $code)) :
+		();
 	return $code;
 }
 
@@ -84,17 +91,49 @@ sub install_accessors : method
 		$me->install_coderef($me->{$type}, $me->$type);
 	}
 	
-	if (defined $me->{handles})
-	{
-		my @pairs = $me->expand_handles;
-		while (@pairs)
-		{
-			my ($target, $method) = splice(@pairs, 0, 2);
-			$me->install_coderef($target, $me->handles($method));
+	if (defined $me->{handles}) {
+		my $shv_data;
+		if ($me->{traits} or $me->{handles_via}) {
+			my $orig_handles = $me->{handles};
+			require Sub::HandlesVia::Toolkit::Plain;
+			$shv_data = 'Sub::HandlesVia::Toolkit::Plain'->clean_spec(
+				$me->{package},
+				$me->{slot},
+				+{%$me},
+			);
+			if ($shv_data) {
+				my $callbacks = 'Sub::HandlesVia::Toolkit::Plain'->make_callbacks(
+					$me->{package},
+					[ $me->reader, $me->writer ],
+				);
+				require Sub::HandlesVia::Handler;
+				$me->{handles} = $orig_handles;
+				my @pairs = $me->expand_handles;
+				while (@pairs) {
+					my ($target, $method) = splice(@pairs, 0, 2);
+					my $handler = 'Sub::HandlesVia::Handler'->lookup($method, $shv_data->{handles_via});
+					$me->install_coderef(
+						$target,
+						$handler->coderef(
+							%$callbacks,
+							target        => $me->{package},
+							method_name   => ref($target) ? '__ANON__' : $target,
+						),
+					);
+				}
+			}
+		}
+		
+		if (!$shv_data) {
+			my @pairs = $me->expand_handles;
+			while (@pairs) {
+				my ($target, $method) = splice(@pairs, 0, 2);
+				$me->install_coderef($target, $me->handles($method));
+			}
 		}
 	}
 	
-	my @return =
+	my @return = map $$_,
 		$me->{is} eq 'ro'   ? ($me->{reader}) :
 		$me->{is} eq 'rw'   ? ($me->{accessor}) :
 		$me->{is} eq 'rwp'  ? ($me->{reader}, $me->{writer}) :
@@ -113,7 +152,9 @@ sub install_coderef
 	if (!ref $target and $target =~ /\A[^\W0-9]\w+\z/)
 	{
 		my $name = "$me->{package}\::$target";
-		$coderef = Sub::Name::subname($name, $coderef) if HAS_SUB_NAME;
+		HAS_SUB_UTIL ? ($coderef = Sub::Util::set_subname($name, $coderef)) :
+		HAS_SUB_NAME ? ($coderef = Sub::Name::subname($name, $coderef)) :
+			();
 		no strict qw(refs);
 		*$name = $coderef;
 		return;
@@ -186,7 +227,9 @@ sub expand_handles
 			
 		if (ref $me->{builder} eq 'CODE')
 		{
-			HAS_SUB_NAME or do { require Sub::Name };
+			HAS_SUB_UTIL or
+			HAS_SUB_NAME or
+			do { require Sub::Util };
 			
 			my $code = $me->{builder};
 			defined($name) && defined($me->{package})
@@ -251,7 +294,7 @@ sub canonicalize_default : method
 	}
 	
 	croak("Invalid default; expected a CODE ref")
-		unless ref $me->{default} ne 'CODE';
+		unless ref $me->{default} eq 'CODE';
 }
 
 sub canonicalize_isa : method
@@ -561,12 +604,12 @@ sub inline_accessor : method
 	
 	if ($coerce eq '$_[1]')  # i.e. no coercion
 	{
-		if (!$me->{trigger} and !$me->{weak_ref})
+		if (!$me->{lazy} and !$me->{trigger} and !$me->{weak_ref})
 		{
 			return sprintf(
 				'(@_ > 1) ? (%s) : %s',
 				$me->inline_access_w( $me->inline_type_assertion('$_[1]') ),
-				$get,
+				$me->inline_get,
 			);
 		}
 		
@@ -576,7 +619,7 @@ sub inline_accessor : method
 			$me->inline_trigger('$_[1]', $get),
 			$me->inline_access_w('$_[1]'),
 			$me->inline_weaken,
-			$me->inline_get,
+			$me->inline_reader,
 		);
 	}
 	
@@ -587,7 +630,7 @@ sub inline_accessor : method
 		$me->inline_trigger('$val', $get),
 		$me->inline_access_w('$val'),
 		$me->inline_weaken,
-		$me->inline_get,
+		$me->inline_reader,
 	);
 }
 
@@ -748,7 +791,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
